"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * 生成器的核心内容，包含生成的方法
 * 2019-09-10 13:20.
 *
 * @author WheelerLee https://github.com/WheelerLee
 * @copyright 2019
 */
const colors_1 = __importDefault(require("colors"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const ejs_1 = __importDefault(require("ejs"));
const Worker_1 = __importDefault(require("./Worker"));
const ora_1 = __importDefault(require("ora"));
const omit_models = [
    'Attach', 'Xt_user', 'Xt_resource', 'Xt_role', 'Xt_dict', 'Xt_role_resource',
    'Xt_user_resource', 'Xt_user_role', 'Xt_setting', 'Msg_send_record', 'Msg_message',
    'Xt_app_version'
]; //需要忽略的模型
class Generator {
    constructor(folderName) {
        this.folderName = folderName;
    }
    /**
     * 生成后台代码
     * @param reset 是否覆盖已经存在的文件生成
     * @param skip 是否跳过依赖添加
     * @param model 是否只生成单个model的增删改查
     */
    generate(reset, skip, model) {
        return __awaiter(this, void 0, void 0, function* () {
            if (model) {
                //先检查是否存在该model
                let models = yield this.getModels();
                if (models.indexOf(model) < 0) {
                    console.log(colors_1.default.red.bold(`无法找到Model：${model}，请检查名称是否准确`));
                    return;
                }
                //reset的情况先删除存在的文件
                if (reset) {
                    const files = [
                        `${process.cwd()}/api/controllers/${this.folderName}/${model}Controller.js`,
                        `${process.cwd()}/views/${this.folderName}/${model.toLowerCase()}/`
                    ];
                    for (let file of files) {
                        fs_extra_1.default.removeSync(file);
                    }
                }
                //检查是否已经存在需要生成的文件
                if (fs_extra_1.default.existsSync(`${process.cwd()}/views/${this.folderName}/${model.toLowerCase()}/`)
                    || fs_extra_1.default.existsSync(`${process.cwd()}/api/controllers/${this.folderName}/${model}Controller/`)) {
                    console.log(colors_1.default.red.bold(`请检查controllers或views文件夹, 已经存在${model}的管理方法`));
                    return;
                } //存在controllers或views文件夹，并且不生成单个的model的情况下会弹出警告，如果有model，则表示生成单个的model的方法
                //确保一些文件夹是否存在
                yield Promise.all([
                    fs_extra_1.default.ensureDirSync(`${process.cwd()}/views/${this.folderName}/`),
                    fs_extra_1.default.ensureDirSync(`${process.cwd()}/api/controllers/${this.folderName}/`)
                ]);
                const spinner = ora_1.default('生成代码').start();
                yield this.generateModel(model);
                spinner.succeed('生成代码');
                spinner.stopAndPersist({
                    text: '代码生成成功'
                });
            }
            else {
                //是否需要重置
                if (reset)
                    this.reset();
                //检查是否创建的位置已经存在文件
                if (fs_extra_1.default.existsSync(`${process.cwd()}/views/${this.folderName}/`)
                    || fs_extra_1.default.existsSync(`${process.cwd()}/api/controllers/${this.folderName}/`)) {
                    console.log(colors_1.default.red.bold(`请检查controllers或views文件夹, ${this.folderName}文件夹已经存在`));
                    return;
                } //存在controllers或views文件夹，并且不生成单个的model的情况下会弹出警告，如果有model，则表示生成单个的model的方法
                const spinner = ora_1.default('生成代码').start();
                yield this.createFolder();
                let models = yield this.getModels();
                for (let model of models) {
                    yield this.generateModel(model);
                }
                yield this.cpAdminModel();
                yield this.generateMockController(models);
                yield this.addPolicies(models);
                yield this.copyBaseFiles();
                yield this.copyAuthFiles();
                yield this.unzipStaticFiles();
                yield this.updateConfigFile();
                spinner.succeed('生成代码');
                if (!skip) { //忽略依赖添加
                    yield this.exec();
                }
                spinner.stopAndPersist({
                    text: '代码生成成功，运行后请访问 http://127.0.0.1:1337/mock/add 添加基础数据'
                });
            }
            // this.core(skip).then((a) => {
            //   spinner.succeed('代码生成成功，运行后请访问 http://127.0.0.1:1337/mock/add 添加基础数据\n');
            // });
        });
    }
    /**
     * 移除历史生成的项目，会移除控制器下admin文件夹等
     */
    reset() {
        //重置需要删除的文件
        const files = [
            `${process.cwd()}/api/controllers/${this.folderName}/`,
            `${process.cwd()}/api/controllers/MockController.js`,
            `${process.cwd()}/api/controllers/StaticController.js`,
            `${process.cwd()}/api/policies/`,
            `${process.cwd()}/api/services/`,
            `${process.cwd()}/assets/admin/`,
            `${process.cwd()}/assets/layui/`,
            `${process.cwd()}/views/${this.folderName}/`
        ];
        for (let file of files) {
            fs_extra_1.default.removeSync(file);
        }
    }
    /**
     * 创建生成代码存放的各级文件夹
     */
    createFolder() {
        return Promise.all([
            fs_extra_1.default.ensureDirSync(`${process.cwd()}/views/${this.folderName}/`),
            fs_extra_1.default.ensureDirSync(`${process.cwd()}/api/controllers/${this.folderName}/`),
            fs_extra_1.default.ensureDirSync(`${process.cwd()}/api/services/`),
            fs_extra_1.default.ensureDirSync(`${process.cwd()}/api/policies/`)
        ]);
    }
    /**
     * 获取项目包含的model，剔除了基本的model
     * @returns string[] 包含的models
     */
    getModels() {
        return fs_extra_1.default.readdir(`${process.cwd()}/api/models/`).then(files => {
            let models = [];
            for (let file of files) {
                if (file.endsWith('.js')) {
                    let modelName = file.replace('.js', '');
                    if (omit_models.indexOf(modelName) < 0) {
                        models.push(modelName);
                    }
                }
            }
            return models;
        });
    }
    /**
     * 生成model的控制器和页面
     * @param fileName model的文件名
     */
    generateModel(fileName) {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log(colors.progress('正在创建模型的操作方法...'));
            let model = require(`${process.cwd()}/api/models/${fileName}`);
            let modelName = fileName.replace('.js', '');
            const common_attr = {
                id: {
                    type: 'number',
                    autoIncrement: true,
                    primaryKey: true
                },
                deleted: {
                    type: 'number',
                    defaultsTo: 0,
                    columnType: 'integer'
                },
                create_user: {
                    type: 'string',
                    maxLength: 250,
                    columnType: 'varchar(250)'
                },
                sorted_num: {
                    type: 'number',
                    defaultsTo: 1,
                    columnType: 'integer'
                }
            }; //通用字段
            model.attributes = Object.assign(model.attributes, common_attr);
            //得到主键的字段
            let primaryKey = null;
            for (let i in model.attributes) {
                let attr = model.attributes[i];
                if (attr['primaryKey']) {
                    primaryKey = i;
                    break;
                }
            }
            if (!primaryKey) {
                console.log(colors_1.default.warn(`model${modelName}没有设置主键，已忽略`));
            }
            else {
                if (omit_models.indexOf(modelName) < 0) { //权限相关的直接生产权限系统
                    yield Worker_1.default.generateController(this.folderName, modelName, model, primaryKey);
                    yield Worker_1.default.generateIndexPage(this.folderName, modelName, model, primaryKey);
                    yield Worker_1.default.generateAddPage(this.folderName, modelName, model, primaryKey);
                }
            }
        });
    }
    /**
     * 移动基础的model到项目中
     */
    cpAdminModel() {
        return __awaiter(this, void 0, void 0, function* () {
            const proRootPath = __dirname + '/..';
            const controllers = yield fs_extra_1.default.readdir(proRootPath + '/templates/models');
            for (let controller of controllers) {
                if (controller.endsWith('.js')) {
                    if (!(yield fs_extra_1.default.pathExists(`${process.cwd()}/api/models/${controller}`))) {
                        // console.log(colors.info('复制' + controller + '到models下'));
                        let ps = proRootPath + '/templates/models/' + controller;
                        yield fs_extra_1.default.writeFile(process.cwd() + '/api/models/' + controller, yield fs_extra_1.default.readFile(ps));
                    }
                    else {
                        // console.log(colors.warn('models下已经存在' + controller));
                    }
                }
            }
        });
    }
    /**
     * 生成mock控制器，包含add方法等
     * @param models 所有的model
     */
    generateMockController(models) {
        return __awaiter(this, void 0, void 0, function* () {
            let str = yield fs_extra_1.default.readFile(__dirname + '/../templates/MockController.ejs'); //读取控制器模板
            let controllerStr = ejs_1.default.render(str.toString(), {
                folderName: this.folderName,
                models: models
            });
            let path = `${process.cwd()}/api/controllers/`;
            yield fs_extra_1.default.ensureDir(path);
            yield fs_extra_1.default.writeFile(`${path}MockController.js`, Buffer.from(controllerStr), { flag: 'w', encoding: 'utf8' });
            // console.log(colors.info('创建MockController'));
        });
    }
    /**
     * 添加请求策略
     * @param models 所有的模型
     */
    addPolicies(models) {
        return __awaiter(this, void 0, void 0, function* () {
            let str = yield fs_extra_1.default.readFile(__dirname + '/../templates/policies.ejs'); //读取policies模板
            let policiesStr = ejs_1.default.render(str.toString(), {
                folderName: this.folderName,
                models: models
            });
            let file_path = process.cwd() + '/config/policies.js';
            yield fs_extra_1.default.writeFile(file_path, Buffer.from(policiesStr), { flag: 'w', encoding: 'utf8' });
            // console.log(colors.info('插入policies成功'));
        });
    }
    /**
     * 复制基础文件到相关目录
     */
    copyBaseFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log(colors.progress('正在复制基础文件到项目...'));
            let proRootPath = __dirname + '/..';
            yield fs_extra_1.default.writeFile(`${process.cwd()}/views/${this.folderName}/layout.ejs`, yield fs_extra_1.default.readFile(`${proRootPath}/templates/layout.ejs`)); //复制layout.ejs
            let policies = yield fs_extra_1.default.readdir(proRootPath + '/templates/policies');
            for (let i in policies) {
                var folder = policies[i];
                if (folder.startsWith('.')) {
                    continue;
                }
                yield fs_extra_1.default.writeFile(process.cwd() + '/api/policies/' + folder, yield fs_extra_1.default.readFile(proRootPath + '/templates/policies/' + folder));
            }
            let services = yield fs_extra_1.default.readdir(proRootPath + '/templates/services');
            for (let i in services) {
                let folder = services[i];
                if (folder.startsWith('.')) {
                    continue;
                }
                yield fs_extra_1.default.writeFile(process.cwd() + '/api/services/' + folder, yield fs_extra_1.default.readFile(proRootPath + '/templates/services/' + folder));
            }
            // console.log(colors.success('复制基础文件到项目成功'));
        });
    }
    /**
     * 复制权限的文件到项目中
     */
    copyAuthFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log(colors.progress('正在复制权限文件到项目...'));
            let proRootPath = __dirname + '/..';
            let controllers = yield fs_extra_1.default.readdir(proRootPath + '/templates/auth/controllers');
            for (let i in controllers) {
                let controller = controllers[i];
                yield fs_extra_1.default.writeFile(process.cwd() + '/api/controllers/' + this.folderName + '/' + controller, yield fs_extra_1.default.readFile(proRootPath + '/templates/auth/controllers/' + controller));
            }
            yield fs_extra_1.default.writeFile(process.cwd() + '/api/controllers/StaticController.js', yield fs_extra_1.default.readFile(proRootPath + '/templates/others/StaticController.js'));
            let views = yield fs_extra_1.default.readdir(proRootPath + '/templates/auth/views');
            for (let i in views) {
                let folder = views[i];
                if (folder.startsWith('.')) {
                    continue;
                }
                let path = process.cwd() + '/views/' + this.folderName + '/' + folder;
                if (yield fs_extra_1.default.pathExists(path)) {
                }
                else {
                    yield fs_extra_1.default.mkdir(path);
                }
                let files = yield fs_extra_1.default.readdir(proRootPath + '/templates/auth/views/' + folder);
                for (let j in files) {
                    let file = files[j];
                    yield fs_extra_1.default.writeFile(process.cwd() + '/views/' + this.folderName + '/' + folder + '/' + file, yield fs_extra_1.default.readFile(proRootPath + '/templates/auth/views/' + folder + '/' + file));
                }
            }
            // console.log(colors.success('复制权限文件到项目成功'));
        });
    }
    /**
     * 解压静态资源文件
     */
    unzipStaticFiles() {
        // console.log(colors.progress('正在解压静态资源...'));
        return fs_extra_1.default.copy(__dirname + '/..' + '/templates/assets', process.cwd() + '/assets/');
        // console.log(colors.success('解压静态资源成功...'));
    }
    /**
     * 替换配置文件
     * @param {String} folderName 文件夹名
     */
    updateConfigFile() {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log(colors.progress('正在替换配置文件...'));
            let proRootPath = __dirname + '/..';
            let pp = proRootPath + '/templates/configs/';
            let op = process.cwd() + '/config/';
            let files = yield fs_extra_1.default.readdir(pp);
            for (let i in files) {
                let file = files[i];
                if (file.endsWith('.js')) {
                    yield fs_extra_1.default.writeFile(op + file, fs_extra_1.default.readFileSync(pp + file));
                }
            }
            // console.log(colors.success('替换配置文件成功'));
        });
    }
    /**
     * 执行shell脚本添加依赖
     */
    exec() {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log(colors.progress('添加项目依赖...'));
            // const spinner = ora('添加项目依赖\n').start();
            const spinner = ora_1.default('添加项目依赖').start();
            let list = require('../templates/command.json').list;
            for (let command of list) {
                spinner.text = '添加项目依赖: ' + command.name;
                yield this.execPromise(command);
            }
            spinner.succeed('添加项目依赖');
        });
    }
    execPromise(command) {
        return new Promise(function (resolve, reject) {
            let process = require('child_process');
            process.exec(command.exec, function () {
                resolve(true);
            });
        });
    }
}
exports.default = Generator;
